using System;
using System.Net;
using System.Text;
using System.Net.Sockets;
using System.Threading;
using System.IO.Ports;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using SecretLabs.NETMF.Hardware;
using SecretLabs.NETMF.Hardware.Netduino;

namespace BertLogic
{
    class RunRobot
    {
        bool cmdListenState;             // whether or not we are listening to the input from the phone
        OutputPort led;                  // indicates the cmdListenState
        OutputPort turnLeft, turnRight;  // controlling the turn signals
        Thread leftBlink, rightBlink;    // blinking the tail-lights
        SerialPort hc06;                 // the BT radio
        byte[] cmdBuffer = new byte[32]; // the input buffer for receiveing commands
        char cmd = ' ';                  // the last command received
        OutputPort A1, A2, B1, B2;       // motor controller ports
        AnalogInput foreRange;           // forward range finder input
        AnalogInput backRange;           // rear range finder input
        InterruptPort button;            // the onboard button to turn on and off accepting command input
        Timer rangeReader;               // the timer for the two range finder inputs

        public RunRobot()
        {
            // set up the bluetooth radio communication
            hc06 = new SerialPort(SerialPorts.COM1, 9600, Parity.None, 8, StopBits.One);
            hc06.ReadTimeout = 1000;
            hc06.Open();

            //use the onboard LED to indicate that the robot is accepting commands
            led = new OutputPort(Pins.ONBOARD_LED, false);

            // set up turn signal lights
            turnLeft = new OutputPort(Pins.GPIO_PIN_D6, false);
            turnRight = new OutputPort(Pins.GPIO_PIN_D7, false);

            //use onboard button to turn on/off accepting commands
            button = new InterruptPort(Pins.ONBOARD_SW1, false, Port.ResistorMode.Disabled, Port.InterruptMode.InterruptEdgeHigh);

            //motor controller output
            A1 = new OutputPort(Pins.GPIO_PIN_D2, false);   //A is the left tires when viewed from the front
            A2 = new OutputPort(Pins.GPIO_PIN_D3, false);
            B1 = new OutputPort(Pins.GPIO_PIN_D4, false);   //B is the right tires when viewed from the front
            B2 = new OutputPort(Pins.GPIO_PIN_D5, false);

            // setup range finder for collision and edge avoidance
            foreRange = new AnalogInput(Cpu.AnalogChannel.ANALOG_5, 100, 0, -1);
            backRange = new AnalogInput(Cpu.AnalogChannel.ANALOG_4, 100, 0, -1);
        }

        public void Start()
        {
            // enable interrupts on BT input
            hc06.DataReceived += hc06_DataReceived;

            //start monitoring for button presses
            button.OnInterrupt += new NativeEventHandler(button_OnInterrupt);

            //turn signal threads setup
            leftBlink = new Thread(leftBlinkLogic);
            leftBlink.Start();
            Thread.Sleep(100);
            leftBlink.Suspend();
            rightBlink = new Thread(rightBlinkLogic);
            rightBlink.Start();
            Thread.Sleep(100);
            rightBlink.Suspend();
            Thread.Sleep(200);
        }
        private void hc06_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            Debug.Print("Data Received");
            if (cmdListenState)
            {
                //read all input
                int count = 0;
                do
                {
                    try
                    {
                        count = hc06.Read(cmdBuffer, 0, cmdBuffer.Length);  // should read on byte at a time.
                    }
                    catch (Exception) { }        // just eat exceptions 
                    if (count > 0)
                    {
                            cmd = (char)cmdBuffer[count-1];
                            executeCmd(cmd);   //execute command immediately
                    }
                } while (count > 0);

            }
            else 
            {
                executeCmd('S');     //double check that the device is not left with the motors running somehow.
            }
        }

        private void button_OnInterrupt(uint data1, uint data2, DateTime time)
        {
            cmdListenState = !cmdListenState;

            if (cmdListenState)
            {
                // setup timer for rangeFinders
                rangeReader = new Timer(new TimerCallback(rangeTimerHandler), null, 0, 100); 
            }
            else
            {
                rangeReader.Dispose();              // drop the timer as we are not processing commansds so we shouldn't be moving
            }

            led.Write(cmdListenState);              // the onboard LED indicates whether Bert is accepting commands
        }


        private void executeCmd(char cmd)
        {
            /*         
            EA  A2  A1  Motor 1         EB  B2  B1  Motor 2 
            0   X   X   Shaft free      0   X   X   Shaft free 
            1   0   0   Shaft locked    1   0   0   Shaft locked 
            1   0   1   Clockwise       1   0   1   Clockwise 
            1   1   0   Anti-clockwise  1   1   0   Anti-clockwise 
            1   1   1   Shaft locked    1   1   1   Shaft locked  
            */
            leftBlink.Suspend();
            rightBlink.Suspend();
            turnRight.Write(false);
            turnLeft.Write(false);
            switch (cmd)
            {
                case 'F':
                    {
                        Debug.Print("Forward");
                        A1.Write(false);    //left forward = false/true
                        A2.Write(true);
                        B1.Write(true);     // right forward = true/false
                        B2.Write(false);
                        break;
                    }
                case 'L':
                    {
                        Debug.Print("Left");
                        leftBlink.Resume();
                        A1.Write(false);    //left forward = false/true
                        A2.Write(true);
                        B1.Write(false);     //right backward = false/true
                        B2.Write(true);
                        break;
                    }
                case 'R':
                    {
                        Debug.Print("Right");
                        rightBlink.Resume();
                        A1.Write(true);     //left backward = true/false
                        A2.Write(false);
                        B1.Write(true);    //right forward = true/false
                        B2.Write(false);
                        break;
                    }
                case 'B':
                    {
                        Debug.Print("Back");
                        A1.Write(true);     //left backward = true/false
                        A2.Write(false);
                        B1.Write(false);    //right backward = false/true
                        B2.Write(true);
                        break;
                    }
                case 'S':
                    {
                        Debug.Print("Stop");
                        A1.Write(true);
                        A2.Write(true);
                        B1.Write(true);
                        B2.Write(true);
                        break;
                    }
                default:
                    {
                        Debug.Print("Undefined input");
                        break;
                    }
            }
        }
        private void rangeTimerHandler(object o)
        {
            bool appStateHold = cmdListenState;
            // normally the device will be listening
            double reading = foreRange.Read();
            if (reading < 20 || reading > 50)
            {
                cmdListenState = false;  // we are not listening for commands during this
                executeCmd('B');
                Thread.Sleep(500);
                executeCmd('L');
                Thread.Sleep(1000);
                executeCmd('S');
                cmdListenState = appStateHold;   //restore listening state
            }
            reading = backRange.Read();
            if (reading > 30)
            {
                cmdListenState = false;   // we are not listening for commands during this
                executeCmd('F');
                Thread.Sleep(500);
                executeCmd('R');
                Thread.Sleep(1000);
                executeCmd('S');
                cmdListenState = appStateHold;   //restore listening state

            }
        }
        private void leftBlinkLogic()
        {
            while (true)
            {
                turnLeft.Write(true);
                Thread.Sleep(300);
                turnLeft.Write(false);
                Thread.Sleep(300);
            }
        }
        private void rightBlinkLogic()
        {
            while (true)
            {
                turnRight.Write(true);
                Thread.Sleep(300);
                turnRight.Write(false);
                Thread.Sleep(300);
            }
        }
    }
}
